;##########################################################################
; Toolbar Template
;##########################################################################

InitToolBar			PROTO :DWORD,:DWORD,:DWORD,:DWORD ; Place in WM_INITDIALOG event
SizeToolBar			PROTO :DWORD,:DWORD ; Call from WM_SIZE event
InitToolBarDividers	PROTO :DWORD ; Place in WM_INITDIALOG event
SizeToolBarDividers	PROTO :DWORD,:DWORD ; Place in WM_SIZE event
SetToolBarTips		PROTO :DWORD ; Place wherever
SetToolBarText		PROTO :DWORD ; Place wherever
ToolBarTips			PROTO :DWORD
SetBmpColor			PROTO :DWORD ; Place in WM_INITDIALOG event After InitToolBar

.CONST
IDC_TOOLBAR_DIVIDER	equ 65534 ; ResID of toolbar divider created automatically by init functions
IDC_MENUBAR_DIVIDER	equ 65533 ; ResID of menubar divider created automatically by init functions

; Flags to set toolbar tips and toolbar text on/off
TOOLBARTIPS_ON		equ TRUE
TOOLBARTIPS_OFF		equ FALSE
TOOLBARTEXT_ON		equ TRUE
TOOLBARTEXT_OFF		equ FALSE

; Toolbar IDs
TB_NEW				equ 50
TB_OPEN				equ 51
TB_ADD				equ 52
TB_EXTRACT			equ 53
TB_VIEW				equ 54
TB_INFO				equ 55
TB_EXIT				equ 56
TB_HELP				equ 57

; ToolBar Tooltips String Table IDs - 1000 more than toolbar resources
TT_NEW				equ 1050
TT_OPEN				equ 1051
TT_ADD				equ 1052
TT_EXTRACT			equ 1053
TT_VIEW				equ 1054
TT_INFO				equ 1055
TT_EXIT				equ 1056
TT_HELP				equ 1057


.DATA
ToolBarTipsFlag		db	TOOLBARTIPS_ON ; Flag to decide to show toolbar tips for buttons - Default Starting Status ON
ToolBarTextFlag		db	TOOLBARTEXT_ON ; Flag to decide to show toolbar text for buttons - Default Starting Status ON
ToolBarTipText 		db 64 dup (0) ; buffer to hold toolbar button tip text for displaying

.DATA?
hToolBar			dd ? ; Handle for ToolBar
hToolBarDivider		dd ? ; Handle for ToolBar Divider
hMenuBarDivider		dd ? ; Handle for ToolBar Menu Divider
ToolBarHeight		dd ?

.CODE


;**************************************************************************
; Initialize the Toolbar Control - Call from WM_INITDIALOG event
; Width & Height of Bitmap Buttons, Ressource ID of Button Bitmap
;**************************************************************************
InitToolBar	PROC	hWin:DWORD, Wd:DWORD, Ht:DWORD, BMPResID:DWORD
	LOCAL bSize:DWORD		; Size of bitmap
	LOCAL tbab:TBADDBITMAP	; Add Bitmap Structure
	LOCAL tbb:TBBUTTON		; Button Structure
	LOCAL hTbBmp:DWORD			; Toolbar bitmap handle
	
	Invoke LoadBitmap,hInstance,BMPResID	; Load bitmap from Resource ID of Bitmap
	mov hTbBmp,eax				; save bitmap handle for later use

	Invoke GetDlgItem, hWin, IDC_MAINTOOLBAR ; Get Handle of Resource ID of Main Toolbar
	mov hToolBar, eax ; Save Handle for later use
    
	invoke SendMessage, hToolBar, TB_BUTTONSTRUCTSIZE, sizeof TBBUTTON, 0	; Set toolbar struct size

	;*****************************************
	; Put width & height of bitmap into DWORD
	;*****************************************
	mov  ecx,Wd  ; loword = bitmap Width
	mov  eax,Ht  ; hiword = bitmap Height
	mov ToolBarHeight, eax ; save toolbar height for later calculations
	shl  eax,16
	mov  ax, cx
	mov bSize, eax

	invoke SendMessage, hToolBar, TB_SETBITMAPSIZE, 0, bSize ; Set bitmap size

	invoke SetBmpColor, hTbBmp
	mov hTbBmp,eax

	mov tbab.hInst, 0
	m2m tbab.nID, hTbBmp
	invoke SendMessage, hToolBar, TB_ADDBITMAP, 7, ADDR tbab ; Number of buttons to create
	invoke SendMessage, hToolBar, TB_SETBUTTONSIZE, 0, bSize ; Set each button size

	; Need to add this to enable all buttons by default
	mov tbb.fsState, TBSTATE_ENABLED
	mov tbb.dwData, 0
	mov tbb.iString, 0
	
	;*********************************
	; Add Each Button And Button Text
	;*********************************
	
	mov tbb.iBitmap,   0  ;; button  ID number
	mov tbb.idCommand, TB_NEW  ;; command ID number
	mov tbb.fsStyle,   TBSTYLE_BUTTON
	.IF ToolBarTextFlag
		mov tbb.iString, CTEXT("New")
	.ENDIF
	invoke SendMessage,hToolBar,TB_ADDBUTTONS,1,ADDR tbb	

	mov tbb.iBitmap,   1  ;; button  ID number
	mov tbb.idCommand, TB_OPEN  ;; command ID number
	mov tbb.fsStyle,   TBSTYLE_BUTTON
	.IF ToolBarTextFlag
		mov tbb.iString, CTEXT("Open")
	.ENDIF
	invoke SendMessage,hToolBar,TB_ADDBUTTONS,1,ADDR tbb	

	mov tbb.iBitmap,   2  ;; button  ID number
	mov tbb.idCommand, TB_ADD  ;; command ID number
	mov tbb.fsStyle,   TBSTYLE_BUTTON
	.IF ToolBarTextFlag
		mov tbb.iString, CTEXT("Add")
	.ENDIF
	invoke SendMessage,hToolBar,TB_ADDBUTTONS,1,ADDR tbb	

	mov tbb.iBitmap,   3  ;; button  ID number
	mov tbb.idCommand, TB_EXTRACT  ;; command ID number
	mov tbb.fsStyle,   TBSTYLE_BUTTON
	.IF ToolBarTextFlag
		mov tbb.iString, CTEXT("Extract")
	.ENDIF
	invoke SendMessage,hToolBar,TB_ADDBUTTONS,1,ADDR tbb	

	mov tbb.iBitmap,   4  ;; button  ID number
	mov tbb.idCommand, TB_VIEW  ;; command ID number
	mov tbb.fsStyle,   TBSTYLE_BUTTON
	.IF ToolBarTextFlag	
		mov tbb.iString, CTEXT("View")
	.ENDIF
	invoke SendMessage,hToolBar,TB_ADDBUTTONS,1,ADDR tbb	

	mov tbb.iBitmap,   5  ;; button  ID number
	mov tbb.idCommand, TB_INFO  ;; command ID number
	mov tbb.fsStyle,   TBSTYLE_BUTTON
	.IF ToolBarTextFlag
		mov tbb.iString, CTEXT("Info")
	.ENDIF
	invoke SendMessage,hToolBar,TB_ADDBUTTONS,1,ADDR tbb	

	mov tbb.iBitmap,   6  ;; button  ID number
	mov tbb.idCommand, TB_EXIT  ;; command ID number
	mov tbb.fsStyle,   TBSTYLE_BUTTON
	.IF ToolBarTextFlag
		mov tbb.iString, CTEXT("Exit")
	.ENDIF
	invoke SendMessage,hToolBar,TB_ADDBUTTONS,1,ADDR tbb	
    
	mov tbb.iBitmap,   7  ;; button  ID number
	mov tbb.idCommand, TB_HELP  ;; command ID number
	mov tbb.fsStyle,   TBSTYLE_BUTTON
	.IF ToolBarTextFlag
		mov tbb.iString, CTEXT("Help")
	.ENDIF
	invoke SendMessage,hToolBar,TB_ADDBUTTONS,1,ADDR tbb	

	; Initialize the toolbar dividers ourself
	Invoke InitToolBarDividers, hWin

	ret
InitToolBar	ENDP

;**************************************************************************
; Resize ToolBar - Call from WM_SIZE
;**************************************************************************
SizeToolBar	PROC	hWin:DWORD, lParam:DWORD
	LOCAL rect:RECT

	invoke GetClientRect,hToolBarDivider,addr rect ; get current size of toolbar
	mov eax, ToolBarHeight
	.if ToolBarTextFlag
		add eax, 22
	.else
		add eax, 8
	.endif
	Invoke MoveWindow, hToolBar, rect.left, rect.top, rect.right, eax, TRUE
	
	Invoke SendMessage, hToolBar, TB_AUTOSIZE, 0, 0
	; Handle the sizing of toolbar dividers ourself
	Invoke SizeToolBarDividers, hWin, lParam
	ret
SizeToolBar	ENDP


;**************************************************************************
; Initialize ToolBar Divider - Call from WM_INITDIALOG event
;**************************************************************************
InitToolBarDividers	PROC	hWin:DWORD
	mov eax, ToolBarHeight
	.if ToolBarTextFlag
		add eax, 22
	.else
		add eax, 8
	.endif
    
    Invoke CreateWindowEx,0,
		CTEXT("STATIC"),NULL,
		WS_VISIBLE+WS_CHILD+SS_ETCHEDHORZ,
		-2,eax,800,2,hWin,IDC_TOOLBAR_DIVIDER,
		hInstance,NULL
		; Create it at -2, 72
	mov hToolBarDivider, eax ; Save ToolBar Divider Handle

	Invoke CreateWindowEx,0,
		CTEXT("STATIC"),NULL,
		WS_VISIBLE+WS_CHILD+SS_ETCHEDHORZ,
		-2,0,800,2,hWin,IDC_MENUBAR_DIVIDER,
		hInstance,NULL
	; Create it at -2, 0
	mov hMenuBarDivider, eax ; Save MenuBar Divider Handle

	ret
InitToolBarDividers	ENDP



;**************************************************************************
; Size ToolBar Divider to bottom of Toolbar - Call from WM_SIZE event
;**************************************************************************
SizeToolBarDividers	PROC	hWin:DWORD, lParam:DWORD
	LOCAL rect:RECT

	invoke GetClientRect,hToolBarDivider,addr rect ; get current size of toolbar
	mov	ebx,lParam
	and	ebx,0FFFFh	; get loword of lParam in ebx
	add ebx, 2 ; add 2 for borders either side of main window/dialog
	;mov	eax,lParam
	;shr eax, 16		; get hiword of lParam in eax
	;sub eax, 80		; subtract value from height of toolbar

	mov eax, ToolBarHeight
	.if ToolBarTextFlag
		add eax, 22
	.else
		add eax, 8
	.endif
    
	invoke MoveWindow,hToolBarDivider,-2,eax,ebx,2,TRUE

	invoke MoveWindow,hMenuBarDivider,-2,0,ebx,2,TRUE


	ret
SizeToolBarDividers	ENDP


;**************************************************************************
; Set ToolBar Tips On/Off - Call from wherever
;**************************************************************************
SetToolBarTips		PROC	ToolTipState:DWORD
	.if ToolTipState ; If set to true = on
		mov ToolBarTipsFlag, TOOLBARTIPS_ON
	.else ; else = off
		mov ToolBarTipsFlag, TOOLBARTIPS_OFF
	.endif	
	ret
SetToolBarTips 		ENDP


;**************************************************************************
; Set ToolBar Button Text On/Off - Call from wherever
;**************************************************************************
SetToolBarText		PROC	ToolTextState:DWORD
	.if ToolTextState ; If set to true = on
		mov ToolBarTextFlag, TOOLBARTEXT_ON
	.else ; else = off
		mov ToolBarTextFlag, TOOLBARTEXT_OFF
	.endif
	ret
SetToolBarText		ENDP


;**************************************************************************
; Process ToolBar Buttons Tooltips - Call from WM_NOTIFY
;**************************************************************************
ToolBarTips			PROC	lParam:DWORD
	.if ToolBarTipsFlag
		mov edi, lParam
		mov eax, (NMHDR PTR [edi]).code
		.if eax ==  TTN_NEEDTEXT
			mov edi, lParam
			mov eax, (NMHDR PTR [edi]).idFrom
			add eax, 1000 ; add 1000 to get tooltip
			Invoke LoadString, hInstance, eax, ADDR ToolBarTipText, 64
			mov edi, lParam
			mov (TOOLTIPTEXT PTR [edi]).lpszText, OFFSET ToolBarTipText
		.endif
	.endif
	ret
ToolBarTips endp



;**************************************************************************
; Used for identifying mask on bitmap and dividing up buttons
;**************************************************************************
SetBmpColor	PROC	hBitmap:DWORD
	LOCAL mDC       :DWORD
	LOCAL hBrush    :DWORD
	LOCAL hOldBmp   :DWORD
	LOCAL hReturn   :DWORD
	LOCAL hOldBrush :DWORD

	invoke CreateCompatibleDC,NULL
	mov mDC,eax

	invoke SelectObject,mDC,hBitmap
	mov hOldBmp,eax

	invoke GetSysColor,COLOR_BTNFACE
	invoke CreateSolidBrush,eax
	mov hBrush,eax

	invoke SelectObject,mDC,hBrush
	mov hOldBrush,eax

	invoke GetPixel,mDC,1,1
	invoke ExtFloodFill,mDC,1,1,eax,FLOODFILLSURFACE

	invoke SelectObject,mDC,hOldBrush
	invoke DeleteObject,hBrush

	invoke SelectObject,mDC,hBitmap
	mov hReturn,eax
	invoke DeleteDC,mDC

	mov eax,hReturn

	ret

SetBmpColor	ENDP
